package de.unisaar.jfap.ex1;

import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Professor with associate students.
 *
 * @author Jörg Steffen, DFKI
 */
public class Professor extends Person {

  private static final Logger logger = LoggerFactory.getLogger(Professor.class);

  // the associated students
  private List<Student> students;

  /**
   * Create a new professor with the given first and last name and an empty list of associated
   * students.
   *
   * @param firstName the first name, not {@code null}
   * @param lastName the last name, not {@code null}
   */
  public Professor(String firstName, String lastName) {

    // just delegate to constructor of superclass
    super(firstName, lastName);

    this.students = new ArrayList<>();
  }

  public List<Student> getStudents() {
    // return copy to protect list from outside changes
    return new ArrayList<>(this.students);
  }

  /**
   * Add the given student to the students associated with this professor. Also add this professor
   * to the professors associated with the given student.
   *
   * @param stud the student to add, not {@code null}
   */
  public void addStudent(Student stud) {
    checkParamsForNonNull(stud);
    logger.debug("adding student {} to {} ...", stud, this);
    if (!this.students.contains(stud)) {
      this.students.add(stud);
      if (!stud.getTeachers().contains(this)) {
        stud.addProfessor(this);
      }
    } else {
      logger.warn("student {} already associated with {}, ignoring", stud, this);
    }
  }

  /**
   * Remove the given student from the students associated with this professor. Also remove this
   * professor from the professors associated with the given student.
   *
   * @param stud the student to remove, not {@code null}
   * @return {@code true} if successful, {@code false} otherwise
   */
  public boolean removeStudent(Student stud) {
    checkParamsForNonNull(stud);
    logger.debug("removing student {} from {} ...", stud, this);
    if (!this.students.remove(stud)) {
      logger.error("student {} is not associated with {}", stud, this);
      return false;
    } else {
      if (stud.getTeachers().contains(this)) {
        stud.removeProfessor(this);
      }
      return true;
    }
  }

  @Override
  public String toString() {
    return String.format("Prof. %s", super.toString());
  }
}
