package de.unisaar.jfap.ex1;

import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Student with associated professors.
 *
 * @author Jörg Steffen, DFKI
 */
public class Student extends Person {

  private static final Logger logger = LoggerFactory.getLogger(Student.class);

  // the associated professors
  private List<Professor> teachers;

  /**
   * Create a new student with the given first and last name and an empty list of associated
   * professors.
   *
   * @param firstName the first name, not {@code null}
   * @param lastName the last name, not {@code null}
   */
  public Student(String firstName, String lastName) {

    // just delegate to constructor of superclass
    super(firstName, lastName);

    this.teachers = new ArrayList<>();
  }

  public List<Professor> getTeachers() {
    // return copy to protect list from outside changes
    return new ArrayList<>(this.teachers);
  }

  /**
   * Add the given professor to the professors associated with this student. Also add this student
   * to the students associated with the given professor.
   *
   * @param prof the professor to add, not {@code null}
   */
  public void addProfessor(Professor prof) {
    checkParamsForNonNull(prof);
    logger.debug("adding {} to student {} ...", prof, this);
    if (!this.teachers.contains(prof)) {
      this.teachers.add(prof);
      if (!prof.getStudents().contains(this)) {
        prof.addStudent(this);
      }
    } else {
      logger.warn("{} already associated with student {}, ignoring", prof, this);
    }
  }

  /**
   * Remove the given professor from the professors associated with this student. Also remove this
   * student from the students associated with given professor.
   *
   * @param prof the professor to remove, not {@code null}
   * @return {@code true} if successful, {@code false} otherwise
   */
  public boolean removeProfessor(Professor prof) {
    checkParamsForNonNull(prof);
    logger.debug("removing {} from student {} ...", prof, this);
    if (!this.teachers.remove(prof)) {
      logger.error("{} is not associated with student {}", prof, this);
      return false;
    } else {
      if (prof.getStudents().contains(this)) {
        prof.removeStudent(this);
      }
      return true;
    }
  }
}
